package ru.dbotthepony.kbox2d.api

import ru.dbotthepony.kstarbound.math.AABB
import ru.dbotthepony.kstarbound.math.Vector2d

typealias b2Pair = Pair<Int, Int>

const val e_nullProxy = -1

/**
 * The broad-phase is used for computing pairs and performing volume queries and ray casts.
 * This broad-phase does not persist pairs. Instead, this reports potentially new pairs.
 * It is up to the client to consume the new pairs and to track subsequent overlap.
 */
interface IBroadPhase : IProxieable, IMovable {
	/**
	 * Call to trigger a re-processing of it's pairs on the next call to UpdatePairs.
 	 */
	fun touchProxy(proxyID: Int)

	/**
	 * Test overlap of fat AABBs.
	 */
	fun testOverlap(proxyIDA: Int, proxyIDB: Int): Boolean

	/**
	 * Get the number of proxies.
	 */
	val proxyCount: Int

	/**
	 * Update the pairs. This results in pair callbacks. This can only add pairs.
	 */
	fun updatePairs(callback: (Any?, Any?) -> Unit)

	/**
	 * Get the height of the embedded tree.
	 */
	val treeHeight: Int

	/**
	 * Get the balance of the embedded tree.
	 */
	val treeBalance: Int

	/**
	 * Get the quality metric of the embedded tree.
	 */
	val treeQuality: Double
}
