package ru.dbotthepony.kstarbound.math

import kotlin.math.cos
import kotlin.math.sin

interface IAngle {
	val pitch: Double
	val yaw: Double
	val roll: Double

	fun matrixX(): Matrix4f {
		val s = sin(pitch).toFloat()
		val c = cos(pitch).toFloat()

		return Matrix4f(
			m11 = c, m12 = -s,
			m21 = s, m22 = c,
		)
	}

	fun matrixY(): Matrix4f {
		val s = sin(yaw).toFloat()
		val c = cos(yaw).toFloat()

		return Matrix4f(
			m00 = c, m02 = s,
			m20 = -s, m22 = c
		)
	}

	fun matrixZ(): Matrix4f {
		val s = sin(roll).toFloat()
		val c = cos(roll).toFloat()

		return Matrix4f(
			m00 = c, m01 = -s,
			m10 = s, m11 = c
		)
	}

	fun matrixXYZ(): Matrix4f {
		return matrixX() * matrixY() * matrixZ()
	}
}

data class Angle(
	override val pitch: Double = 0.0,
	override val yaw: Double = 0.0,
	override val roll: Double = 0.0,
) : IAngle
