package ru.dbotthepony.kstarbound.util

import com.google.gson.TypeAdapter
import com.google.gson.stream.JsonReader
import com.google.gson.stream.JsonWriter

interface IStringSerializable {
	fun match(name: String): Boolean
	fun write(out: JsonWriter)
}

class CustomEnumTypeAdapter<T : Enum<T>>(private val clazz: Array<T>) : TypeAdapter<T>() {
	override fun write(out: JsonWriter, value: T) {
		if (value is IStringSerializable)
			value.write(out)
		else
			out.value(value.name)
	}

	override fun read(`in`: JsonReader): T {
		val str = `in`.nextString().uppercase()

		for (value in clazz) {
			if (value is IStringSerializable && value.match(str) || value.name == str) {
				return value
			}
		}

		throw IllegalArgumentException("${clazz[0]::class.java.name} does not have value for $str")
	}
}
